//**************************************
//* Class Location :                   *
//*  Class used for storing location   *
//*  of an object		       *
//**************************************

void error(char *);     // outputs an error message then exits.

class Location
{
    int LineStart,	// Line number where the entity starts   
        LineEnd,	// Line number where the entity ends.    
        ColumnStart,	// Column number of the beginning of the entity.   
        ColumnEnd;	// Column number of the end of the entity.   

    int OK( void );     // Checks some properties of the object.

  public:

    // Several Constructors for initialisation of Location Object.
    // The entity for the two first constructors is set to end at the same
    // point than its beginnning.

    // First constructor sets the Location to the beginning of the Text.
    Location( void );

    // Second constructor sets the Location to line Line and column Column.
    // If Column is ommited, it is assumed that you are talking about the
    // column 1.

    Location( int Line , int Column = 1 );

    // Last constructor for the ones who knows about the beginning and the
    // end of the entity referenced.
    Location( int First_Line, int First_Column,
	      int Last_Line, int Last_Column);

    Location( const Location& );

    // Set the point in the text where the entity ends.
    void SetEnd( int Line, int Column );

    // Move the beginning of the Location to the place indicated.
    void Move( int Line , int Column );

    // Advance the start position to n columns.
    void Tab( int distance );

    // Return the start position to next line.
    void Cr( void );

    void Select( int length );

    void Advance( void );

    void CrSelect( void );

    // Overloading of the operator + for computing new Location
    // The Location Object resulting begins at the first Location
    // ( The most little position in the text) and ends at the
    // last location (The furthest position).
    friend Location operator + ( const Location& , const Location& );

    // friend operation that allows to write the Location of the entity on
    // the output stream.
    friend ostream& operator << ( ostream& cout, Location there );
} ;

//**
//** Several structure used inside the lex and yacc structure
//**

//** LocInt : concatenation of a integer and a location Object.
typedef struct
{
  int      Int;
  Location *Loc;
} LocInt;

//** LocBool : concatenation of a boolean and a location Object.
typedef struct
{
  int       Bool;
  Location *Loc;
} LocBool;

//** LocInt : concatenation of a integer and a location Object.
typedef struct
{
  double    Real;
  Location *Loc;
} LocDouble;

//** LocStr : concatenation of a String and a location Object.
typedef struct
{
  String   *Str;
  Location *Loc;
} LocStr;

//** LocChar : concatenation of a character and a location Object.
typedef struct
{
  char      Char;
  Location *Loc;
} LocChar;

//*******************************************
//* Class ErrorEntry                        *
//*    This Class describes an error emited *
//* during the analysis of the text.        *
//*******************************************

enum ErrorKind {Warning,          // This entry is a warning.
		Error,            // This entry is a real error.
		Fatal,            // This entry is a spurious error.
		Internal,         // This error is because of the compiler.
		Operating_system, // This error is because of the operating system.
		Unknown};         // We don't know why an error has been generated.

class ErrorEntry
{
  Location   place;            // Where it has happened.
  int        number;           // The number of error encountered.
  ErrorKind  kind;             // The type of error (see upper).
  char      *comment;          // A precision about the error.

  public:
    // Two constructors to initialize an ErrorEntry object.

    // First constructor, give the location of the error and its identification
    ErrorEntry( Location, int, ErrorKind );

    // Second constructor, just add a commend to the error.
    ErrorEntry( Location, int, ErrorKind, char * );

    // the friend operator << that allows to output the content of an
    // ErrorEntry object into an output stream.
    friend ostream operator << ( ostream& cout, ErrorEntry TheError);
} ;

//*******************************************
//* Class ErrorStorage                      *
//*    This class describes the object that *
//* holds all of the errors and warning     *
//* produced by the analysis phase.         *
//*******************************************

class ErrorStorage
{

} ;
